#include <QObject>
#include <QProcess>
#include <QList>
#include <QGraphicsRectItem>

class QGraphicsScene;
class SvnModelPriv;

class SvnEntry {
public:
    uint revision;
    QString author;
    QString msg;
    uint time;

    SvnEntry() :revision(0) {}
    bool operator<(const SvnEntry& e) {
        return this->revision < e.revision;
    }
};
class SvnModel;
class SvnGraphicsItem : public QGraphicsRectItem {
private:
    SvnModel* model;
protected:
    void hoverEnterEvent(QGraphicsSceneHoverEvent* event);
public:
    const SvnEntry entry;
    SvnGraphicsItem(const SvnEntry& e, QGraphicsScene* s, SvnModel* m)
        :QGraphicsRectItem(0,s), model(m), entry(e) {
    }
    void setPosParams(double zoom, double offset);
};

class SvnModel : public QObject {
Q_OBJECT
private:
    QString svndir;
    QGraphicsScene* scene;
    SvnModelPriv* p;
    QList<SvnGraphicsItem*> entries;
    QMap<QString, QColor> colors;
    uint mintime;
    uint maxtime;
    double zoom;
    double minzoom;
    double maxzoom;
    double offset;
    int width;
    int lastx, lasty;
    bool hadinput;

    void updateLimits();
    bool updateView(int x1, int y1, int x2, int y2);
    void updateCoords();
private slots:
    void parseInputIncrementally();
    void processFinished(int exitCode, QProcess::ExitStatus exitStatus);

protected:
    bool eventFilter(QObject *obj, QEvent *event);
 
public:
    SvnModel(QGraphicsScene* scene);
    ~SvnModel();
    void addEntry(const SvnEntry&);
    void setZoom(double z);
    void handleClick(SvnGraphicsItem* item);
public slots:
    void show(const QString& svndir);
signals:
    void itemActivated(const QString& url);
};
